import { ICredentials } from "../interfaces";
/**
 * Returns configuration from the credentials file.
 * @returns The contents of the credentials file if exists.
 */
export declare function getCredentials(): ICredentials;
/**
 * Saves the credentials file to disk.
 * @param credentials - The credential object that needs to be saved to disk.
 */
export declare function saveCredentials(credentials: ICredentials): void;
/**
 * Checks if a credentials file exists.
 * @returns A boolean containing the value if the credentials file exists or not.
 */
export declare function credentialsFileExists(): boolean;
/**
 * Creates the path to the credentials file if needed.
 */
export declare function createCredentialsPathIfNeeded(): void;
/**
 * Gets an Access token from StackPath and save it to disk and returns it.
 * @returns {Promise<string>} - The access_token that was the result of the oauth2 exchange.
 */
export declare function getAccessToken(): Promise<string>;
/**
 * Check if current Access token is expired.
 * @returns {Promise<boolean>} If true, the token has expired.
 */
export declare function isAccessTokenExpired(): Promise<boolean>;
