"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const node_fetch_1 = require("node-fetch");
const Auth = require("./auth");
const constants_1 = require("../constants");
const Log = require("./log");
/**
 * Perform a http call to the server.
 * @param {string} method The http method used to perform the call
 * @param {string} resource - The endpoint relative to the hostname, be sure to prefix with a forward slash '/'.
 * @param [body] - The optional payload that will be sent to the server.
 * @param {boolean} throwErrors - If false, handle the error yourself, timeout errors will still be thrown. Defaults to true.
 * @returns {Promise<Response>} - The response of the server.
 */
async function request(method, resource, body) {
    try {
        const response = await node_fetch_1.default(`${constants_1.STACKPATH_HOST}${resource}`, {
            method,
            headers: {
                "content-type": "application/json",
                authorization: `Bearer ${Auth.getCredentials().access_token}`
            },
            body: body ? JSON.stringify(body) : undefined
        });
        Log.logVerbose(`HTTP Request ${method} ${constants_1.STACKPATH_HOST}${resource} ${response.status}`);
        return response;
    }
    catch (error) {
        throw new Error(`An error occurred when connecting to StackPath host ${constants_1.STACKPATH_HOST}.`);
    }
}
exports.request = request;
