"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const deploy_1 = require("../api/services/deploy");
const Auth = require("../api/services/auth");
const constants_1 = require("../api/constants");
const Log = require("../api/services/log");
const LogLevel = Log.LogLevel;
/**
 * Class performing the deployment command '$ sp-serverless deploy'.
 */
class Deploy extends command_1.Command {
    /**
     * Runs CLI command.
     */
    async run() {
        this.parsedFlags = this.parse(Deploy).flags;
        if (!!this.parsedFlags && this.parsedFlags.verbose) {
            process.env.STACKPATH_LOG_LEVEL = LogLevel.VERBOSE || LogLevel.INFO;
        }
        if (!!this.parsedFlags && this.parsedFlags.force) {
            process.env.STACKPATH_FORCE = this.parsedFlags.force;
        }
        Log.logVerbose(`Checking if access token expired?`);
        try {
            if (await Auth.isAccessTokenExpired()) {
                cli_ux_1.default.action.start("Refreshing access token...");
                await Auth.getAccessToken();
                cli_ux_1.default.action.stop("Refreshing access token done.");
            }
            else {
                Log.logVerbose(`Access token is not expired.`);
            }
        }
        catch (e) {
            cli_ux_1.default.error(e.message);
        }
        const deploy = new deploy_1.Deploy();
        try {
            await deploy.deployScripts({
                force: Boolean(process.env.STACKPATH_FORCE)
            });
        }
        catch (e) {
            cli_ux_1.default.log(e.message);
        }
    }
}
Deploy.description = `Deploys the scripts in the working directory according to its ${constants_1.STACKPATH_CONFIGFILE_PATH} configuration file.`;
Deploy.examples = [`$ sp-serverless deploy`];
Deploy.flags = {
    help: command_1.flags.help({ char: "h" }),
    verbose: command_1.flags.boolean({
        char: "v",
        description: "Turns on verbose logging. Defaults to false",
        default: false
    }),
    force: command_1.flags.boolean({
        char: "f",
        description: "Force recreation of scripts if they do not exist. Defaults to false",
        default: false
    })
};
Deploy.args = [];
exports.default = Deploy;
